
//#define SERVER

//#define SERVER_RECV

#include <stdio.h>
#include <signal.h>
#include <pthread.h>

#include "TXTLog.h"
CTXTLog *g_pMo = NULL;
CTXTLog *g_pMt = NULL;
CTXTLog *g_pRpt = NULL;
CTXTLog *g_pLog = NULL;

#ifdef SERVER
#include "GSMPGServer.h"
#else
#include "GSMPClient.h"
#endif

#include "GSMPPkg.h"

CConfig config;

#ifdef SERVER
int server(void);
void* ServerSendMoThread(CGSMPGServer *pServer);
void* ServerRecvMtThread(CGSMPGServer *pServer);
#else
int client(void);
void* ClientSendMtThread(CGSMPClient *pClient);
void* ClientFileSendMtThread(CGSMPClient *pClient);
void* ClientRecvMoThread(CGSMPClient *pClient);
void* ClientRecvRptThread(CGSMPClient *pClient);
#endif 

void CreateMt(CGSMPSubmit &mt, int *n);
void CreateMo(CGSMPDeliver &mo);
CGSMPPkg mPkg;

int g_nTotal = 0;

int main(int argc, char **argv)
{
	signal(SIGPIPE, SIG_IGN);
#ifdef SERVER
	server();
#else
	g_pMo = new CTXTLog("moClient");
	g_pMt = new CTXTLog("mtClient");

	client();
#endif
	cout << "run error" << endl;
	return 0;
}


#ifdef SERVER
int server(void)
{
	config.LoadConfig("./server.xml");
	CGSMPGServer *pServer = NULL;
	
	pServer = new CGSMPGServer();
	
	if (!pServer->GSMPInit())
	{
		cout << "server init error" << endl;
		delete pServer;
		return -1;
	}

	pthread_attr_t pAttr;
	pthread_attr_init(&pAttr);
	pthread_attr_setdetachstate(&pAttr,  PTHREAD_CREATE_DETACHED);
	pthread_t iThrRecv = 0;

	int ret = pthread_create(&iThrRecv, &pAttr, \
		(void *(*) (void *))ServerRecvMtThread, (void *)pServer);
	if(ret != 0)
	{
		printf("====>START ServerRecvMtThread THREAD ERROR!\n");
		delete pServer;
		return -1;
	}
	else
		printf("=====>START ServerRecvMtThreadT THREAD ID:%ld SUCCEED!\n", iThrRecv);

	ret = pthread_create(&iThrRecv, &pAttr, \
		(void *(*) (void *))ServerSendMoThread, (void *)pServer);
	if(ret != 0)
	{
		printf("====>START ServerSendMoThread THREAD ERROR!\n");
		delete pServer;
		return -1;
	}
	else
		printf("=====>START ServerSendMoThread THREAD ID:%ld SUCCEED!\n", iThrRecv);

	while (true)
	{
		usleep(1000 * 1000 * 10);
	}
	
	pServer->GSMPFinal();
	delete pServer;
	return 0;
}

void* ServerRecvMtThread(CGSMPGServer *pServer)
{
	while (true)
	{
		CGSMPSubmit mt;
	
		int nRet = pServer->GSMPRecvSubmit(&mt, "90001", 60);
		if (nRet == 0)
		{
			cout << "=====MT=====" << endl;
			cout << "user: " << mt.m_sDestTermId << endl;
			cout << "cont: " << mt.m_sMsgCont << endl;
		}
		else
		{
			cout << "recv error, return = " << nRet << endl;
		}
		
		usleep(100);
	}

	return NULL;
}

void* ServerSendMoThread(CGSMPGServer *pServer)
{
	while (true)
	{
		CGSMPDeliver mo;
		mo.m_Head.nCommandId = GSMP_DELIVER;
		mo.m_Head.nSequence = mPkg.GetSeqno();
		CreateMo(mo);
		char cMsgID[50];
		memset(cMsgID, 0, sizeof(cMsgID));
		int nRet = pServer->GSMPSendDeliver(&mo, cMsgID, "90001");
		//nRet = pServer->GSMPSendDeliver(&mo, cMsgID, "912341");
		//nRet = pServer->GSMPSendDeliver(&mo, cMsgID, "912342");
		if (nRet == 0)
			cout << "send succeed, msgid = " << cMsgID << endl;
		else
			cout << "send error" << endl;

		//usleep(1000 * 1000 * 2);		
		usleep(10);		
	}

	return NULL;
}

#else

int client(void)
{
	config.LoadConfig("./client.xml");
	CGSMPClient *pClientSend = NULL;
	CGSMPClient *pClientRecv = NULL;
	
	pClientSend = new CGSMPClient();
	pClientRecv = new CGSMPClient();
	
	if (!pClientSend->GSMPInit(CLIENT_MODEL_SEND))
	{
		cout << "client init send error" << endl;
		delete pClientRecv;
		delete pClientSend;
		return -1;
	}

	sleep(10);

	if (!pClientRecv->GSMPInit(CLIENT_MODEL_RECV))
	{
		cout << "client init recv error" << endl;
		delete pClientRecv;
		delete pClientSend;
		return -1;
	}

	pthread_attr_t pAttr;
	pthread_attr_init(&pAttr);
	pthread_attr_setdetachstate(&pAttr,  PTHREAD_CREATE_DETACHED);
	pthread_t iThrRecv = 0;

	int ret = 0;
	
	ret = pthread_create(&iThrRecv, &pAttr, \
		(void *(*) (void *))ClientSendMtThread, (void *)pClientSend);
	if(ret != 0)
	{
		printf("====>START ClientSendMtThread THREAD ERROR!\n");
		delete pClientRecv;
		delete pClientSend;
		return -1;
	}
	else
		printf("=====>START ClientSendMtThread THREAD ID:%ld SUCCEED!\n", iThrRecv);

	ret = pthread_create(&iThrRecv, &pAttr, \
		(void *(*) (void *))ClientRecvMoThread, (void *)pClientRecv);
	if(ret != 0)
	{
		printf("====>START ClientRecvMoThread THREAD ERROR!\n");
		delete pClientRecv;
		delete pClientSend;
		return -1;
	}
	else
		printf("=====>START ClientRecvMoThread THREAD ID:%ld SUCCEED!\n", iThrRecv);

	ret = pthread_create(&iThrRecv, &pAttr, \
		(void *(*) (void *))ClientRecvRptThread, (void *)pClientRecv);
	if(ret != 0)
	{
		printf("====>START ClientRecvRptThread THREAD ERROR!\n");
		delete pClientRecv;
		delete pClientSend;
		return -1;
	}
	else
		printf("=====>START ClientRecvRptThread THREAD ID:%ld SUCCEED!\n", iThrRecv);

	while (true)
	{
		usleep(1000 * 1000 * 10);
	}

	pClientRecv->GSMPFinal();
	pClientSend->GSMPFinal();
	delete pClientRecv;
	delete pClientSend;
	return 0;
}

void* ClientSendMtThread(CGSMPClient *pClient)
{
	while (true)
	{
		CGSMPSubmit mt;
		mt.m_Head.nCommandId = GSMP_SUBMIT;
		mt.m_Head.nSequence = mPkg.GetSeqno();

		int nCount = 0;
		CreateMt(mt, &nCount);
		if (nCount > 0)
		{
			char cMsgID[50];
			memset(cMsgID, 0, sizeof(cMsgID));
			int nRet = pClient->GSMPSendSubmit(&mt, cMsgID);
			char cResult[2+1]={0x00};
			if (nRet == 0)
			{
				strcpy(cResult,"00");
				cout << "send succeed, msgid = " << cMsgID << endl;
			}
			else
			{
				strcpy(cResult,"11");
				cout << "send error" << endl;
			}
			string logStr =  mt.m_sDestTermId+"|"+mt.m_sMsgCont+"|"+mt.m_sNeedReport+"|"+cResult+"|";
			g_pMo->AddLog(logStr);
		}else{
			
		}

		usleep(10*1000*1000);
	}

	return NULL;
}

void* ClientFileSendMtThread(CGSMPClient *pClient)
{
	CGSMPSubmit mt;
	mt.m_Head.nCommandId = GSMP_SUBMIT;
	int nCount = 0;
	CreateMt(mt, &nCount);
	
	FILE *pFile = fopen("./src.txt", "r");
	if (pFile == NULL)
	{
		cout << "open file error" << endl;
		return NULL;
	}

	char cMobile[20];
	char cMsgCont[200];
	char cBuf[1024];
	while (!feof(pFile))
	{
		memset(cBuf, 0, sizeof(cBuf));
		memset(cMobile, 0, sizeof(cMobile));
		memset(cMsgCont, 0, sizeof(cMsgCont));
		
		if (fgets(cBuf, sizeof(cBuf), pFile) == NULL)
			continue;
		else if (cBuf[0] == ' ')
			continue;
		{
			//memset(cBuf, 0, sizeof(cBuf));
			//fgets(cBuf, sizeof(cBuf), pFile);
			cout << "cBuf:" << cBuf << endl;
			for (int i=0; i<sizeof(cBuf); i++)
			{
				if (cBuf[i] == ' ') cBuf[i] = '\0';
			}
			if (strlen(cBuf) < 11)
				continue;

			strcpy(cMobile, cBuf);
			cout << "cMobile:" << cMobile << "|" << endl;

			memset(cBuf, 0, sizeof(cBuf));
			fgets(cBuf, sizeof(cBuf), pFile);
			for (int i=0; i<sizeof(cBuf); i++)
			{
				if (cBuf[i] == ' ') cBuf[i] = '\0';
			}
			strcpy(cMsgCont, cBuf);
			cout << "cMsgCont:" << cMsgCont << "|" << endl;
		}
			
		mt.m_Head.nSequence = mPkg.GetSeqno();
		mt.m_sMsgCont = cMsgCont;
		mt.m_sSrcTermId = "";
		mt.m_sDestTermId = cMobile;
		mt.m_sChargeTermId = cMobile;
		char cMsgID[50];
		memset(cMsgID, 0, sizeof(cMsgID));
		int nRet = 1;

		while (nRet != 0)
		{
			memset(cMsgID, 0, sizeof(cMsgID));
			nRet = pClient->GSMPSendSubmit(&mt, cMsgID);
			if (nRet != 0)	
				sleep(1);
			cout << "send result:" << nRet << endl;
		}
		char cResult[2+1]={0x00};
		if (nRet == 0)
		{
			strcpy(cResult,"00");
			cout << "send succeed, msgid = " << cMsgID << endl;
		}
		else
		{
			strcpy(cResult,"11");
			cout << "send error" << endl;
		}
		string logStr =  mt.m_sDestTermId+"|"+mt.m_sMsgCont+"|"+mt.m_sNeedReport+"|"+cResult+"|";
		g_pMo->AddLog(logStr);

		//usleep(10);
		usleep(1000 * 10);
	}

	if (pFile != NULL)
		fclose(pFile);
	cout << "send succeed" << endl;
	return NULL;
}

void* ClientRecvMoThread(CGSMPClient *pClient)
{
	while(true)
	{
		CGSMPDeliver mo;
	
		char cMsgID[50];
		memset(cMsgID, 0, sizeof(cMsgID));
		int nRet = pClient->GSMPRecvDeliver(&mo, (char *)cMsgID, 60);
		char cResult[2+1]={0x00};
		if (nRet == 0)
		{
			strcpy(cResult,"00");
			cout << "=====MO=====" << endl;
			cout << "user: " << mo.m_sSrcTermId << endl;
			cout << "cont: " << mo.m_sMsgCont << endl;
		}
		else
		{
			strcpy(cResult,"11");
			cout << "recv error, return = " << nRet << endl;
		}
		string strLog = mo.m_sSrcTermId+"|"+mo.m_sMsgCont+"|"+cResult+"|";
		g_pMo->AddLog(strLog);
		//usleep(100);
		usleep(100);
	}

	return NULL;
}

void* ClientRecvRptThread(CGSMPClient *pClient)
{
	while(true)
	{
		CGSMPReport rpt;
	
		char cMsgID[50];
		memset(cMsgID, 0, sizeof(cMsgID));
		int nRet = pClient->GSMPRecvReport(&rpt, (char *)cMsgID, 60);
		if (nRet == 0)
		{
			cout << "=====REPORT=====" << endl;
			cout << "user: " << rpt.m_sDestTermId << endl;
			cout << "msgid: " << rpt.m_sMsgId << endl;
			cout << "submit msgid: " << rpt.m_sSubmitMsgId << endl;
		}
		else
		{
			cout << "recv error, return = " << nRet << endl;
		}
		
		//usleep(100);
		usleep(1000 * 1000 * 5);
	}

	return NULL;
}


#endif

void CreateMt(CGSMPSubmit &mt, int *nCount)
{
	CConfig conf;
	conf.LoadConfig("./mt_sms.xml");
	char cTotal[20];
	memset(cTotal, 0, sizeof(cTotal));
	sprintf(cTotal, "%010d", g_nTotal);
	int n = atoi(conf.GetMsgByName("SENDCOUNT").c_str());//Դ
	
	if(g_nTotal>=n){
		*nCount=0;
		return ;
	}else
	*nCount = n;
	
	mt.m_sMsgType = "10";//Ϣ
	mt.m_sMsgId = "";//Ϣк
	mt.m_sSrcGWId = "";//ԴCPرʶ
	mt.m_sDestGWId = "";//ĿOPرʶ
	//mt.m_sSrcTermId = "51850100";//Դ
	mt.m_sSrcTermId = conf.GetMsgByName("SRCTERMID");//Դ
	mt.m_sChargeTermId = conf.GetMsgByName("DESTTERMID");//Դ
	mt.m_sChargeTermType = "0";//ƷѺͣ0-ʵֻ 1-α
	mt.m_sDestTermCount = "1";//ĿֻĿǰ֧50
	//mt.m_sDestTermId = "13511111111";//Ŀֻ룬֮Զŷָ
	mt.m_sDestTermId = conf.GetMsgByName("DESTTERMID");//Դ
	mt.m_sDestTermType = "0";//Ŀֻͣ0-ʵֻ 1-α
	//mt.m_sServiceId = "CPLLCX";//ҵ
	mt.m_sServiceId = conf.GetMsgByName("SERVICEID");//Դ
	//mt.m_sFeeType = "0";//Ʒ
	mt.m_sFeeType = conf.GetMsgByName("FEETYPE");//Դ
	//mt.m_sFeeCode = "100";//ƷѴ
	mt.m_sFeeCode = conf.GetMsgByName("FEECODE");//Դ
	mt.m_sGivenValue = "100";//	ͨûΪͻ,СͨûΪⶥ,λΪ
	mt.m_sMTF = "0";//MTԭ:
	//mt.m_sMsgLevel = "10";//ȼ
	mt.m_sMsgLevel = conf.GetMsgByName("MSGLEVEL");//Դ
	mt.m_sValidTime = "";//Чʱ
	mt.m_sAtTime = "";//Ϣʱ͵ʱ䣬Ϊգʾ̷͸öϢ
	mt.m_sNeedReport = "1";//ǷҪ״̬ 
	mt.m_sTPpid = "0";//GSMЭ͡ϸǽοGSM03.40е9.2.3.9
	mt.m_sTPudhi = "0";//GSMЭ͡ϸǽοGSM03.40е9.2.3.23
	mt.m_sMsgFmt = "15";//Ϣıʽ
	mt.m_sMsgContType = "0";//ͣ0ı1ҪBASE64
	//mt.m_sMsgCont = "Ϣ";//Ϣ
	//mt.m_sMsgCont += cTotal;
	mt.m_sMsgCont = conf.GetMsgByName("MSGCONTENT");//Դ
	mt.m_sLinkId = "123456789";//㲥MTļȨϢ
	g_nTotal++;
}

void CreateMo(CGSMPDeliver &mt)
{
	g_nTotal++;
	char cTotal[20];
	memset(cTotal, 0, sizeof(cTotal));
	sprintf(cTotal, "%010d", g_nTotal);

	mt.m_sMsgType = "1";//Ϣ
	mt.m_sMsgId = "";//Ϣк
	mt.m_sSrcGWId = "1988";//ԴCPرʶ
	mt.m_sDestGWId = "90001";//ĿOPرʶ
	mt.m_sSrcTermId = "13910412572";//Դ
	mt.m_sSrcTermType = "0";//ƷѺͣ0-ʵֻ 1-α
	mt.m_sDestTermId = "581818";//Ŀֻ룬֮Զŷָ
	mt.m_sServiceId = "HELP";//ҵ
	mt.m_sMsgLevel = "0";//ȼ
	mt.m_sMsgFmt = "15";//Ϣıʽ
	mt.m_sMsgContType = "0";//ͣ0ı1ҪBASE64
	mt.m_sMsgCont = "Ϣ";//Ϣ
	mt.m_sMsgCont += cTotal;
	mt.m_sLinkId = "123456789";//㲥MTļȨϢ
}

